#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Created By Murray(murray) on 2021/1/28 上午10:39
--------------------------------------------------
"""
import uuid


def val_of(data, idx, default=None, typ=None, is_hex=False):
    try:
        if isinstance(data, (str, int, float, bytes, uuid.UUID)):
            val = data
        elif isinstance(data, (list, dict, tuple)):
            val = data[idx]
        else:
            val = getattr(data, idx, default)
        if typ is not None:
            if typ in [int, float]:
                val = typ(val or 0)
            elif is_hex and isinstance(val, uuid.UUID):
                val = val.hex
            if val is not None:
                val = typ(val)
    except (Exception,):
        val = None
    val = default if val is None else val
    return val



#!/usr/bin/python
# -*- coding: utf8 -*-

from __future__ import print_function

import unittest

from stormer import Requester


class TestClient(unittest.TestCase):
    def test_get_server(self):
        # init Requester instance
        requester = Requester(
            server_url="https://www.baidu.com",
            # redis_url="redis://127.0.0.1:6379/0",
            # timeout=30,  # in seconds
            # headers={"Content-Type": "text/html;charset=utf8"},
            encoding='utf8'
        )

        # open debug
        requester.set_debugging()

        # register request function
        requester.register(
            action="get",
            func="bd_index",
            uri="/"
        )

        # execute function
        rlt = requester.bd_index()
        # print(rlt.data)
        print(rlt.resp)

        self.assertEqual(rlt.status_code, 200)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Created By Murray(m18527) on 2019/12/13 13:57
"""
from __future__ import absolute_import, unicode_literals

SUCCESS = 0

GONE_AWAY = -1
UNKNOWN_ERROR = 10000
BAD_REQUEST = 10001
ERROR_PARAMS = 10002

MSG = {
    SUCCESS: '正常。',
    GONE_AWAY: '服务器开小差了，请稍后再试。',
    UNKNOWN_ERROR: '未知错误。',
    BAD_REQUEST: '错误请求。',
    ERROR_PARAMS: '参数错误。',
}



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Created By Murray(m18527) on 2019/12/13 13:57
"""
import json as _json
import logging
import os
from collections import namedtuple
from urllib.parse import urljoin

import chardet
import requests
from urllib3 import encode_multipart_formdata

from .redis_utils import get_redis_cache

logger = logging.getLogger(__name__)

VERSION = "0.1.2"

DEBUG = os.getenv("DEBUG", False)

Resp = namedtuple("Resp", ["status_code", "content", "reason"])

META_CACHE_KEY = "REQUESTER:META:{PARAMS_HASH}"
CONTENT_CACHE_KEY = "REQUESTER:CONTENT:{PARAMS_HASH}"


class RespResult(object):
    """
    Used for packing request response
    """

    def __init__(self, response=None, url=None, action=None, params_hash=None, redis_cache=None, encoding=None):
        if not isinstance(response, requests.Response):
            msg = "Param <response> should be object of requests.Response, but {} found.".format(type(response))
            logger.warning(msg)
            raise Exception(msg)
        self.response = response
        self.ok = None
        self.reason = None
        self.content = None
        self.headers = None
        self.encoding = encoding
        self.status_code = None
        self.url = url
        self.action = action
        self.params_hash = params_hash
        self.redis_cache = redis_cache
        self._init()

    def _init(self):
        if self.encoding:
            self.response.encoding = self.encoding
        self.encoding = self.response.encoding
        self.ok = self.response.ok
        self.reason = self.response.reason
        self.content = self.response.content
        self.text = self.response.text
        self.headers = self.response.headers.__repr__()

        self.status_code = self.response.status_code

    @staticmethod
    def guest_encoding(text):
        """The apparent encoding, provided by the chardet library."""
        return chardet.detect(text)['encoding']

    @property
    def bytes(self):
        return self.content

    @property
    def json(self):
        return self.response.json()

    @property
    def data(self):
        try:
            _data = self.json
        except (Exception,):
            _data = self.text
            if not _data:
                _data = self.bytes
        return _data

    @property
    def resp(self):
        """
        parse resp to api resp
        """
        if 200 <= self.status_code < 300:
            return Resp(status_code=self.status_code, content=self.data, reason=None)
        if isinstance(self.reason, bytes):
            try:
                reason = self.reason.decode(self.encoding or "utf-8")
            except UnicodeDecodeError:
                reason = self.reason.decode('iso-8859-1')
        else:
            reason = self.reason
        logger.error("The request failed, http status code: {status}, content: {content}, reason: {reason}".format(
            status=self.status_code, content=self.data, reason=self.reason))
        return Resp(status_code=self.status_code, content=self.data, reason=reason)

    def set_cache(self, timeout):
        if not (timeout > 0 and self.redis_cache and self.params_hash) \
                or not str(self.action).upper() == "GET":
            return None
        meta_data = {
            "ok": self.ok,
            "reason": self.reason,
            "text": self.text,
            "headers": self.headers,
            "encoding": self.encoding,
            "status_code": self.status_code,
            "url": self.url,
            "action": self.action,
        }
        meta_key = META_CACHE_KEY.format(PARAMS_HASH=self.params_hash)
        content_key = CONTENT_CACHE_KEY.format(PARAMS_HASH=self.params_hash)
        self.redis_cache.set(meta_key, _json.dumps(meta_data), ex=timeout)
        self.redis_cache.set(content_key, self.content, ex=timeout)

    @classmethod
    def from_cache(cls, params_hash, action, redis_conn):
        if not (redis_conn and action and action.upper() == "GET"):
            return None
        meta_key = META_CACHE_KEY.format(PARAMS_HASH=params_hash)
        content_key = CONTENT_CACHE_KEY.format(PARAMS_HASH=params_hash)
        meta_data = redis_conn.get(meta_key)
        content = redis_conn.get(content_key)
        if not (meta_data and content):
            return None
        result = cls()
        for key, value in _json.loads(meta_data).items():
            setattr(result, key, value)
        setattr(result, "content", content)
        logger.debug("Fetch data from the cache by hash {}.".format(params_hash))
        return result


class Requester(object):
    """Requester is request Class which base on questions, it be used for send request."""
    debug = False

    @staticmethod
    def set_debugging():
        if not Requester.debug:
            global logger
            logger = logging.getLogger("requester")
            handler = logging.StreamHandler()
            handler.setFormatter(logging.Formatter(
                "%(asctime)s - [%(levelname)s] [%(filename)s:%(lineno)d] - %(message)s"))
            logger.addHandler(handler)
            logger.setLevel(logging.DEBUG)
            Requester.debug = True

    def __init__(self, server_url, headers=None, proxies=None, encoding=None,
                 redis_url=None, redis_nodes=None, redis_password='', timeout=0):
        """
        Init Requester
        :param server_url string 'https://xxx.com'
        :param headers dict {'Content-Type': 'application/json'}
        :param proxies dict {"http": None, "https": None}
        :param redis_url string 'redis://:<password>@<host>:<port>/<db>'
        :param redis_nodes string 'ip:port,ip:port'
        :param redis_password string  'redis password'
        """
        self.headers = headers
        self.proxies = proxies or {"http": None, "https": None}
        self.server_url = server_url
        self.redis_url = redis_url
        self.redis_nodes = redis_nodes
        self.redis_password = redis_password
        self.timeout = timeout
        self.encoding = encoding
        self.cache = None
        self.apis = []

        if self.redis_url or self.redis_nodes:
            self.cache = get_redis_cache(redis_url=redis_url, redis_nodes=redis_nodes, password=self.redis_password)

    @staticmethod
    def gen_md5(text, salt='_stormer_requester_'):
        import hashlib
        md = hashlib.md5()
        md.update("{}{}".format(text, salt).encode("utf-8"))
        res = md.hexdigest()
        return res

    @classmethod
    def build_params_hash(cls, url, params, **kwargs):
        new_params = {k: v for k, v in (params or {}).items()}
        new_params.update({"url": url})
        new_params.update(kwargs)
        new_params = _json.dumps(new_params, sort_keys=True)
        return str(cls.gen_md5(new_params)).upper()

    @staticmethod
    def _path_url(url, path_params):
        if path_params and isinstance(path_params, dict):
            url = url.format(**path_params)
        return url

    def _bind_func(self, pre_url, action, timeout=0):
        def req(path_params=None, params=None, data=None, json=None, files=None, forms=None, **kwargs):
            url = self._path_url(pre_url, path_params)
            params_hash, resp_result = None, None
            if self.cache and timeout > 0:
                params_hash = self.build_params_hash(url, params, **kwargs)
                resp_result = RespResult.from_cache(params_hash, action, self.cache)
            if not resp_result:
                resp = self._do_request(
                    action=action, url=url, params=params, data=data, json=json,
                    files=files, forms=forms, **kwargs
                )
                resp_result = RespResult(
                    response=resp, url=url, action=action,
                    redis_cache=self.cache, params_hash=params_hash,
                    encoding=self.encoding
                )
                resp_result.set_cache(timeout)
            return resp_result

        return req

    def _add_path(self, action, uri, func_name, timeout=0):
        action = action.upper()
        func_name = func_name.lower()
        assert func_name not in self.apis, u"Duplicate function {}.".format(func_name)
        url = urljoin(self.server_url, uri)
        setattr(self, func_name, self._bind_func(url, action, timeout=timeout))
        self.apis.append(func_name)
        logger.debug("Requester register method: {method}, path: {url}".format(method=func_name, url=url))
        return getattr(self, func_name)

    @staticmethod
    def _func_name(func):
        try:
            func_name = func.__name__
        except (Exception,):
            func_name = str(func)
        return func_name

    def register(self, action, func, uri, timeout=0):
        func_name = self._func_name(func)
        return self._add_path(action, uri, func_name, timeout=timeout or self.timeout)

    def _headers(self, headers):
        """combine headers"""
        if headers and isinstance(headers, dict):
            if self.headers:
                for key, value in self.headers.items():
                    if key in headers:
                        continue
                    headers[key] = value
        else:
            headers = self.headers
        return headers

    def _proxies(self, proxies):
        """combine proxies"""
        if proxies and isinstance(proxies, dict):
            if self.proxies:
                for key, value in self.headers.items():
                    if key in proxies:
                        continue
                    proxies[key] = value
        else:
            proxies = self.proxies
        return proxies

    def _do_request(self, action, url, params=None, data=None, json=None, files=None, forms=None, **kwargs):
        assert url, "request url can't be blank."
        kwargs["headers"] = self._headers(kwargs.get("headers"))
        kwargs["proxies"] = self._proxies(kwargs.get("proxies"))
        if isinstance(forms, dict):
            if isinstance(data, dict):
                forms.update(data)
            encode_data = encode_multipart_formdata(forms)
            data = encode_data[0]
            if isinstance(kwargs["headers"], dict):
                kwargs["headers"]["Content-Type"] = encode_data[1]
            else:
                kwargs["headers"] = {"Content-Type": encode_data[1]}

        if action.upper() == "GET":
            return requests.get(url, params=params, **kwargs)
        elif action.upper() == "POST":
            return requests.post(url, data=data, json=json, files=files, **kwargs)
        elif action.upper() == "PUT":
            return requests.put(url, data=data, json=json, files=files, **kwargs)
        elif action.upper() == "DELETE":
            return requests.delete(url, **kwargs)
        elif action.upper() == "OPTIONS":
            return requests.options(url, **kwargs)
        else:
            msg = "Not Support %s." % action.upper()
            logger.error(msg)
            raise Exception(msg)


if DEBUG:
    Requester.set_debugging()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Created By Murray(Murray) on 2021/1/12 22:08
--------------------------------------------------
pip install redis-py-cluster==2.1.0
"""
import logging

import redis
from rediscluster import RedisCluster

logger = logging.getLogger(__name__)


def spilt_cluster_nodes(nodes):
    """构建redis cluster node"""
    if not nodes:
        return []
    nodes = nodes.split(",")
    _nodes = []
    for node in nodes:
        if not node:
            continue
        _ip_port = node.split(":")
        if len(_ip_port) != 2:
            raise Exception("redis cluster nodes param error, you need setting nodes like ip:7000,ip:7002")
        _nodes.append({
            'host': _ip_port[0],
            'port': _ip_port[1],
            'db': 0
        })
    return _nodes


def redis_cluster(nodes, password=''):
    _cache = None
    try:
        _cache = RedisCluster(
            startup_nodes=nodes,
            password=password,
            decode_responses=True,
            skip_full_coverage_check=True
        )
    except (Exception,) as e:
        logger.error('Init RedisCluster error {}'.format(e))
    return _cache


class RedisClusterCache(object):
    def __init__(self, nodes, password=''):
        self.nodes = nodes
        if not isinstance(self.nodes, list):
            self.spilt_cluster_nodes()
        self.connect = redis_cluster(
            nodes=self.nodes,
            password=password
        )

    def spilt_cluster_nodes(self):
        """构建redis cluster node"""
        if isinstance(self.nodes, list):
            return
        self.nodes = spilt_cluster_nodes(nodes=self.nodes)

    def get_info(self):
        """获取redis集群info信息"""
        # 查看info信息, 返回dict
        dic = self.connect.cluster_info()
        return dic or {}

    def get_has_aof(self):
        """查看aof是否打开"""
        _config = self.connect.config_get('appendonly')
        return _config

    def set(self, key, value, timeout=None, **kwargs):
        """设置缓存"""
        return self.connect.set(key, value, ex=timeout, **kwargs)

    def get(self, key):
        """获取缓存"""
        _val = self.connect.get(key)
        if isinstance(_val, bytes):
            _val = _val.decode(encoding='utf8')
        return _val

    def delete(self, key):
        """删除缓存"""
        return self.connect.delete(key)

    def expire(self, key, timeout):
        """设置过期时间"""
        return self.connect.expire(key, timeout)

    def persist(self, key):
        """取消过期时间"""
        return self.connect.persist(key)

    def ttl(self, key):
        """获取过期时间"""
        return self.connect.ttl(key)


def get_redis_cache(redis_url, redis_nodes, password=''):
    _cache = None
    if redis_url:
        try:
            _cache = redis.Redis(connection_pool=redis.ConnectionPool.from_url(redis_url))
        except ConnectionError:
            logger.warning("Redis connection pool is max number of clients reached, now disconnect.")
            if _cache:
                _cache.connection_pool.disconnect()
    elif redis_nodes:
        _cache = RedisClusterCache(nodes=redis_nodes, password=password)
    if not _cache:
        raise Exception("Init redis cache fail.")
    return _cache


if __name__ == '__main__':
    import time

    _redis_nodes = "103.131.169.104:8000,103.131.169.104:8001,103.131.169.104:8002," \
                   "103.131.169.104:8003,103.131.169.104:8004,103.131.169.104:8005"
    _password = "SXsE3jHZPtYhcYMO"
    cache = RedisClusterCache(nodes=_redis_nodes, password=_password)
    print(cache.get_info())
    print(cache.get_has_aof())
    cache.set("key1", "value1", timeout=2)
    time.sleep(1)
    print("1 ==> ", cache.get("key1"))
    cache.expire("key1", 3)
    time.sleep(2)
    print("2 ==> ", cache.get("key1"))
    print("3 ==> ", cache.ttl("key1"))
    cache.persist("key1")
    print("4 ==> ", cache.ttl("key1"))
    cache.delete("key1")
    print("5 ==> ", cache.get("key1"))



import os

from setuptools import find_packages, setup

import stormer

with open(os.path.join(os.path.dirname(__file__), 'README.md'), "r", encoding='utf-8') as fh:
    LONG_DESCRIPTION = fh.read()

DESCRIPTION = (
    'this is a requester for requests.'
)

CLASSIFIERS = [
    'Programming Language :: Python',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content :: CGI Tools/Libraries',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: Implementation :: CPython',
]
KEYWORDS = [
    'stormer', 'requester', 'redis', 'requests'
]

setup(
    name='stormer',
    version=stormer.__version__,
    maintainer='Murray',
    maintainer_email='sunglowrise@qq.com',
    url='https://github.com/sunglowrise/stormer/',
    download_url='https://github.com/sunglowrise/stormer/',
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    license='MIT',
    platforms='Platform Independent',
    keywords=KEYWORDS,
    classifiers=CLASSIFIERS,
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    python_requires=">=3.5",
    install_requires=[
        'requests>=2.22.0',
        'redis>=3.3.11',
        'redis-py-cluster>=2.1.0'
    ],
)
